@@ -1,7 +1,9 @@
 import io
+import pathlib
 from britive.britive import Britive
 from .helpers.config import ConfigManager
 from .helpers.credentials import FileCredentialManager, EncryptedFileCredentialManager
+from .helpers.split import profile_split
 import json
 import click
 import csv
@@ -14,6 +16,7 @@ from pathlib import Path
 from datetime import datetime
 import os
 import sys
+import jmespath
 
 
 default_table_format = 'fancy_grid'
@@ -141,9 +144,17 @@ class BritiveCli:
 
         if self.output_format == 'json':
             click.echo(json.dumps(data, indent=2, default=str))
+        elif self.output_format == 'list-profiles':
+            for row in data:
+                click.echo(self.list_separator.join([self.escape_profile_element(x) for x in row.values()]))
         elif self.output_format == 'list':
             for row in data:
-                click.echo(self.list_separator.join(row.values()))
+                if isinstance(row, dict):
+                    click.echo(self.list_separator.join([json.dumps(x, default=str) for x in row.values()]))
+                elif isinstance(row, list):
+                    click.echo(self.list_separator.join([json.dumps(x, default=str) for x in row]))
+                else:
+                    click.echo(row)
         elif self.output_format == 'csv':
             fields = list(data[0].keys())
             output = io.StringIO()
@@ -167,7 +178,7 @@ class BritiveCli:
 
     def user(self):
         self.login()
-        username = self.b.my_access.whoami()['user']['username']
+        username = self.b.my_access.whoami()['username']
         alias = self.tenant_alias
         output = f'{username} @ {self.tenant_name}'
         if alias != self.tenant_name:
@@ -200,9 +211,20 @@ class BritiveCli:
                     self.list_separator = '/'
                     row.pop('Description')
                     row.pop('Type')
+                    if profile['2_part_profile_format_allowed']:
+                        row.pop('Environment')
                 data.append(row)
+
+        # set special list output if needed
+        if self.output_format == 'list':
+            self.output_format = 'list-profiles'
+
         self.print(data)
 
+        # and set it back
+        if self.output_format == 'list-profiles':
+            self.output_format = 'list'
+
     def list_applications(self):
         self.login()
         self._set_available_profiles()
@@ -255,12 +277,14 @@ class BritiveCli:
                             'app_description': app['appDescription'],
                             'env_name': env['environmentName'],
                             'env_id': env['environmentId'],
+                            'env_short_name':  env['alternateEnvironmentName'],
                             'env_description': env['environmentDescription'],
                             'profile_name': profile['profileName'],
                             'profile_id': profile['profileId'],
                             'profile_allows_console': profile['consoleAccess'],
                             'profile_allows_programmatic': profile['programmaticAccess'],
-                            'profile_description': profile['profileDescription']
+                            'profile_description': profile['profileDescription'],
+                            '2_part_profile_format_allowed': app['requiresHierarchicalModel']
                         }
                         data.append(row)
             self.available_profiles = data
@@ -279,7 +303,7 @@ class BritiveCli:
         if app_type in ['AWS', 'AWS Standalone']:
             return printer.AwsCloudCredentialPrinter(
                 console=console,
-                mode=mode,
+                mode=mode or self.config.aws_default_checkout_mode(),  # handle the aws default_checkout_mode here
                 profile=profile,
                 credentials=credentials,
                 silent=silent,
@@ -319,20 +343,37 @@ class BritiveCli:
         self.login()
         parts = self._split_profile_into_parts(profile)
 
-        self.b.my_access.checkin_by_name(
+        ids = self._convert_names_to_ids(
             profile_name=parts['profile'],
             environment_name=parts['env'],
             application_name=parts['app']
         )
 
+        transaction_id = None
+        for profile in self.b.my_access.list_checked_out_profiles():
+            if profile['environmentId'] == ids['environment_id'] and profile['papId'] == ids['profile_id']:
+                transaction_id = profile['transactionId']
+                break
+        if not transaction_id:
+            raise ValueError(f'no checked out profile found for the given profile')
+
+        self.b.my_access.checkin(
+            transaction_id=transaction_id
+        )
+
     def _checkout(self, profile_name, env_name, app_name, programmatic, blocktime, maxpolltime, justification):
         try:
             self.login()
 
-            return self.b.my_access.checkout_by_name(
+            ids = self._convert_names_to_ids(
                 profile_name=profile_name,
                 environment_name=env_name,
-                application_name=app_name,
+                application_name=app_name
+            )
+
+            return self.b.my_access.checkout(
+                profile_id=ids['profile_id'],
+                environment_id=ids['environment_id'],
                 programmatic=programmatic,
                 include_credentials=True,
                 wait_time=blocktime,
@@ -359,9 +400,11 @@ class BritiveCli:
 
     def _split_profile_into_parts(self, profile):
         profile_real = self.config.profile_aliases.get(profile, profile)
-        parts = profile_real.split('/')
+        parts = profile_split(profile_real)
+        if len(parts) == 2:  # handle shortcut for profiles where the app and environment name are the same
+            parts = [parts[0], parts[0], parts[1]]
         if len(parts) != 3:
-            raise click.ClickException('Provided profile string does not have the required 3 parts.')
+            raise click.ClickException('Provided profile string does not have the required parts.')
         parts_dict = {
             'app': parts[0],
             'env': parts[1],
@@ -377,6 +420,8 @@ class BritiveCli:
         response = None
         self.verbose_checkout = verbose
 
+        self._validate_justification(justification)
+
         if mode == 'awscredentialprocess':
             self.silent = True  # the aws credential process CANNOT output anything other than the expected JSON
             # we need to check the credential process cache for the credentials first
@@ -413,7 +458,7 @@ class BritiveCli:
             credentials = response['credentials']
 
         # this handles the --force-renew flag
-        # lets check to see if the we should checkin this profile first and check it out again
+        # lets check to see if we should checkin this profile first and check it out again
         if self._should_check_force_renew(app_type, force_renew, console):
             expiration = datetime.fromisoformat(credentials['expirationTime'].replace('Z', ''))
             now = datetime.utcnow()
@@ -425,7 +470,7 @@ class BritiveCli:
                 credential_process_creds_found = False  # need to write new creds to cache
                 credentials = response['credentials']
 
-        if alias:  # do this down here so we know that the profile is valid and a checkout was successful
+        if alias:  # do this down here, so we know that the profile is valid and a checkout was successful
             self.config.save_profile_alias(alias=alias, profile=profile)
 
         if mode == 'awscredentialprocess' and not credential_process_creds_found:
@@ -481,7 +526,8 @@ class BritiveCli:
             backend=backend
         )
 
-    def viewsecret(self, path, blocktime, justification,maxpolltime):
+    def viewsecret(self, path, blocktime, justification, maxpolltime):
+        self._validate_justification(justification)
         self.login()
 
         try:
@@ -511,7 +557,8 @@ class BritiveCli:
         # and finally print the secret data
         self.print(value)
 
-    def downloadsecret(self, path, blocktime, justification,maxpolltime, file):
+    def downloadsecret(self, path, blocktime, justification, maxpolltime, file):
+        self._validate_justification(justification)
         self.login()
 
         try:
@@ -551,9 +598,21 @@ class BritiveCli:
             self._set_available_profiles()
         profiles = []
         for p in self.available_profiles:
-            profiles.append(f"{p['app_name']}/{p['env_name']}/{p['profile_name']}")
+            profile = self.escape_profile_element(p['app_name'])
+            profile += '/'
+
+            if not p['2_part_profile_format_allowed']:
+                profile += self.escape_profile_element(p['env_name'])
+                profile += '/'
+
+            profile += self.escape_profile_element(p['profile_name'])
+            profiles.append(profile)
         Cache().save_profiles(profiles)
 
+    @staticmethod
+    def escape_profile_element(element):
+        return element.replace('/', '\\/')
+
     @staticmethod
     def cache_clear():
         Cache().clear()
@@ -562,13 +621,19 @@ class BritiveCli:
         self.config.update(section=section, field=field, value=value)
 
     def request_submit(self, profile, justification):
+        self._validate_justification(justification)
         self.login()
         parts = self._split_profile_into_parts(profile)
 
-        self.b.my_access.request_approval_by_name(
+        ids = self._convert_names_to_ids(
             profile_name=parts['profile'],
             environment_name=parts['env'],
-            application_name=parts['app'],
+            application_name=parts['app']
+        )
+
+        self.b.my_access.request_approval(
+            profile_id=ids['profile_id'],
+            environment_id=ids['environment_id'],
             block_until_disposition=False,
             justification=justification
         )
@@ -577,16 +642,135 @@ class BritiveCli:
         self.login()
         parts = self._split_profile_into_parts(profile)
 
-        self.b.my_access.withdraw_approval_request_by_name(
+        ids = self._convert_names_to_ids(
             profile_name=parts['profile'],
             environment_name=parts['env'],
             application_name=parts['app']
         )
 
+        self.b.my_access.withdraw_approval_request(
+            profile_id=ids['profile_id'],
+            environment_id=ids['environment_id']
+        )
+
     def clear_gcloud_auth_key_files(self):
         self.config.clear_gcloud_auth_key_files()
 
+    def api(self, method, parameters={}, query=None):
+        self.login()
 
+        # clean up parameters - need to load json as dict if json string is provided and handle file inputs
+        computed_parameters = {}
+        open_file_keys = []
+        for key, value in parameters.items():
+            computed_key = key.replace('-', '_')
+            computed_value = value
+
+            if value.lower() == 'none':
+                computed_value = None
+
+            if value.startswith('file://'):
+                filepath = value.replace('file://', '')
+                path = pathlib.Path(filepath)
+                with open(str(path), 'r') as f:
+                    computed_value = f.read().strip()
+
+            if value.startswith('fileb://'):
+                filepath = value.replace('fileb://', '')
+                path = pathlib.Path(filepath)
+                computed_value = open(str(path), 'rb')
+                open_file_keys.append(computed_key)
+
+            try:
+                computed_parameters[computed_key] = json.loads(computed_value)
+            except json.JSONDecodeError:
+                computed_parameters[computed_key] = computed_value
+            except Exception:  # not sure what else we would do so just default to the value provided
+                computed_parameters[computed_key] = computed_value
+
+        # determine the sdk method we need to execute, starting at the base Britive class
+        func = self.b
+        try:
+            for m in method.split('.'):
+                func = getattr(func, m)
+        except Exception as e:
+            raise click.ClickException(f'invalid method {method} provided.')
+
+        # execute the method with the computed parameters
+        response = func(**computed_parameters)
+
+        # close any files we opened due to fileb:// prefix
+        for key in open_file_keys:
+            try:
+                computed_parameters[key].close()
+            except Exception:
+                pass
+
+        # output the response, optionally filtering based on provided jmespath query/search
+        self.print(jmespath.search(query, response) if query else response)
+
+    # yes - this method exits in b.my_access as _get_profile_and_environment_ids_given_names
+    # but we are doing additional business logic here to enhance the cli experience so there is
+    # a need to duplicate some of this logic (although we are doing additional work here so the logic is
+    # not 100% duplicated)
+    def _convert_names_to_ids(self, profile_name: str, environment_name: str, application_name: str) -> dict:
+        # set the available profiles if this is not already done
+        self._set_available_profiles()
+
+        # do some sanitization just in case
+        profile_name = profile_name.lower().strip()
+        environment_name = environment_name.lower().strip()
+        application_name = application_name.lower().strip()
+
+        found_profiles = {}
+
+        # collect relevant profile/environment combinations to which the identity is entitled
+        for profile in self.available_profiles:
+            if profile['app_name'].lower() != application_name:  # kick out all the unmatched applications
+                continue
+            if profile['profile_name'].lower() != profile_name:  # kick out the unmatched profiles
+                continue
+
+            # if we get here we know we are on a record that is the right app and right profile
+
+            found_profile_id = profile['profile_id']
+
+            if found_profile_id not in found_profiles.keys():
+                found_profiles[found_profile_id] = []
+
+            # load up multiple options
+            env_options = [
+                profile['env_name'].lower(),
+                profile['env_id'].lower(),
+                profile['env_short_name'].lower()
+            ]
+
+            if environment_name in env_options:
+                found_profiles[found_profile_id].append(profile['env_id'])
+
+        # let's first check to ensure we have only 1 profile
+        if len(found_profiles.keys()) == 0:
+            raise click.ClickException('no profile found with the provided application, environment, and profile names')
+        if len(found_profiles.keys()) > 1:
+            raise click.ClickException('multiple matching profiles found - cannot determine which profile to use')
+
+        # and now we can check to ensure we have only 1 environment
+        found_profile_id = list(found_profiles.keys())[0]
+        possible_environments = found_profiles[found_profile_id]
+        if len(possible_environments) == 0:
+            raise click.ClickException('no profile found with the provided application, environment, and profile names')
+        if len(possible_environments) > 1:
+            raise click.ClickException('multiple matching profiles found - cannot determine which profile to use')
+
+        return {
+            'profile_id': found_profile_id,
+            'environment_id': possible_environments[0]
+        }
+
+    @staticmethod
+    def _validate_justification(justification: str):
+        if justification and len(justification) > 255:
+            raise ValueError('justification cannot be longer than 255 characters.')
 
 
 
