@@ -11,6 +11,7 @@ from .commands.secret import secret as group_secret
 from .commands.cache import cache as group_cache
 from .commands.request import request as group_request
 from .commands.clear import clear as group_clear
+from .commands.api import api as command_api
 import sys
 import os
 
@@ -23,9 +24,9 @@ def safe_cli():
         cli()
     except Exception as e:
         if debug:
-            raise click.ClickException(str(e))
+            raise e
         else:
-            raise click.ClickException(str(e)) from None
+            raise e from None
 
 
 # this is the "main" app - it really does nothing but print the overview/help section
@@ -48,6 +49,7 @@ cli.add_command(group_secret)
 cli.add_command(group_cache)
 cli.add_command(group_request)
 cli.add_command(group_clear)
+cli.add_command(command_api)
 
 
 if __name__ == "__main__":
