@@ -1,7 +1,8 @@
 import click
 from ..helpers.build_britive import build_britive
 from ..options.britive_options import britive_options
-from ..completers.profile import profile_completer
+from ..helpers.profile_argument_dectorator import click_smart_profile_argument
+
 
 @click.group()
 def request():
@@ -12,7 +13,7 @@ def request():
 @request.command()
 @build_britive
 @britive_options(names='justification,tenant,token,passphrase,federation_provider')
-@click.argument('profile', shell_complete=profile_completer)
+@click_smart_profile_argument
 def submit(ctx, justification, tenant, token, passphrase, federation_provider, profile):
     """Submit a request to checkout a profile.
 
@@ -33,7 +34,7 @@ def submit(ctx, justification, tenant, token, passphrase, federation_provider, p
 @request.command()
 @build_britive
 @britive_options(names='tenant,token,passphrase,federation_provider')
-@click.argument('profile', shell_complete=profile_completer)
+@click_smart_profile_argument
 def withdraw(ctx, tenant, token, passphrase, federation_provider, profile):
     """Withdraw a request to checkout a profile.
 
@@ -45,4 +46,4 @@ def withdraw(ctx, tenant, token, passphrase, federation_provider, profile):
 
     ctx.obj.britive.request_withdraw(
         profile=profile
-    )
\ No newline at end of file
+    )
