@@ -5,6 +5,6 @@ option = click.option(
     '--justification', '-j',
     default=None,
     show_default=True,
-    help='Justification for the checkout approval process, if the profile checkout requires approval.'
+    help='Justification for the checkout approval process, if the profile checkout requires approval.',
 )
 
